@@ -1,7 +1,7 @@
 #
 #       images.R
 #
-#      $Revision: 1.177 $     $Date: 2023/01/15 02:01:31 $
+#      $Revision: 1.180 $     $Date: 2023/11/04 04:39:23 $
 #
 #      The class "im" of raster images
 #
@@ -311,7 +311,7 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
         if(jtype == "given")
           warning("Argument j ignored")
         W <- raster %orifnull% as.owin(x)
-        M <- as.mask.psp(as.psp(i), W=W, ...)
+        M <- psp2mask(as.psp(i), W=W, ...)
         xM <- x[M, drop=drop]
         if(is.im(xM)) xM <- spatstat.linnet::linim(i, xM)
         return(xM)
@@ -1086,10 +1086,6 @@ quantilefun.im <- function(x, ..., type=1) {
   quantilefun(f, type=type)
 }
   
-integral <- function(f, domain=NULL, ...) {
-  UseMethod("integral")
-}
-
 integral.im <- function(f, domain=NULL, weight=NULL, ...) {
   verifyclass(f, "im")
   if(!is.null(weight)) {
@@ -1368,17 +1364,3 @@ fillNA <- function(x, value=0) {
   return(x)
 }
 
-
-## probability integral transformation
-## aka histogram equalisation
-## aka transformation to uniformity
-
-transformquantiles <- function(X, uniform=FALSE, reverse=FALSE, ...) {
-  if(!uniform && !reverse) return(X)
-  o <- order(X[])
-  V <- X
-  n <- length(o)
-  if(uniform) V[][o] <- (seq_len(n) - 0.5)/n
-  if(reverse) V[][o] <- V[][rev(o)]
-  return(V)
-}
